package np.something.services;

import javax.transaction.Transactional;

import np.something.DTO.CustomerDto;
import np.something.Exceptions.CustomerNotFoundException;
import np.something.Exceptions.JwtException;
import np.something.model.Customer;
import np.something.model.UserRole;
import np.something.repositories.CustomerRepository;
import np.something.security.JwtProvider;
import np.something.util.validation.ValidatorUtil;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;

@Service
public class CustomerService implements UserDetailsService {
    private final CustomerRepository customerRepository;
    private final ValidatorUtil validatorUtil;
    private final PasswordEncoder passwordEncoder;
    private final JwtProvider jwtProvider;

    public CustomerService(CustomerRepository customerRepository,
                           ValidatorUtil validatorUtil, PasswordEncoder passwordEncoder, JwtProvider jwtProvider) {
        this.customerRepository = customerRepository;
        this.validatorUtil = validatorUtil;
        this.passwordEncoder = passwordEncoder;
        this.jwtProvider = jwtProvider;
    }

    @Transactional
    public Customer findCustomer(Long id) {
        return customerRepository.findById(id).orElseThrow(() -> new CustomerNotFoundException(id));
    }

    @Transactional
    public List<Customer> findAllCustomers() {
        return customerRepository.findAll();
    }

    @Transactional
    public Customer addCustomer(String username, String password) {
        Customer customer = new Customer(username, passwordEncoder.encode(password));
        validatorUtil.validate(customer);
        return customerRepository.save(customer);
    }

    @Transactional
    public Customer addCustomer(String username, String password, UserRole role) {
        Customer customer = new Customer(username, passwordEncoder.encode(password), role);
        validatorUtil.validate(customer);
        return customerRepository.save(customer);
    }

    @Transactional
    public Customer updateCustomer(Long id, String username, String password) {
        Customer customer = findCustomer(id);
        customer.setUsername(username);
        customer.setPassword(passwordEncoder.encode(password));
        validatorUtil.validate(customer);
        return customerRepository.save(customer);
    }

    @Transactional
    public Customer deleteCustomer(Long id) {
        Customer customer = findCustomer(id);
        customerRepository.delete(customer);
        return customer;
    }

    @Transactional
    public void deleteAllCustomers() {
        customerRepository.deleteAll();
    }

    public Customer findByUsername(String username) {
        return customerRepository.findByUsername(username);
    }

    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        final Customer customerEntity = findByUsername(username);
        if (customerEntity == null) {
            throw new UsernameNotFoundException(username);
        }
        return new org.springframework.security.core.userdetails.User(
                customerEntity.getUsername(), customerEntity.getPassword(), Collections.singleton(customerEntity.getRole()));
    }

    public String loginAndGetToken(CustomerDto customerDto) {
        try {
            final Customer customer = findByUsername(customerDto.getUsername());
            if (customer == null) {
                throw new Exception("Login not found" + customerDto.getUsername());
            }
            if (!passwordEncoder.matches(customerDto.getPassword(), customer.getPassword())) {
                throw new Exception("User not found" + customer.getUsername());
            }
            return jwtProvider.generateToken(customer.getUsername());
        }
        catch (Exception e) {
            return null;
        }
    }

    public UserDetails loadUserByToken(String token) throws UsernameNotFoundException {
        if (!jwtProvider.isTokenValid(token)) {
            throw new JwtException("Bad token");
        }
        final String userLogin = jwtProvider.getLoginFromToken(token)
                .orElseThrow(() -> new JwtException("Token is not contain Login"));
        return loadUserByUsername(userLogin);
    }
}
