package np.something.services;

import javax.transaction.Transactional;

import np.something.DTO.PostDto;
import np.something.Exceptions.PostNotFoundException;
import np.something.model.Customer;
import np.something.model.Post;
import np.something.repositories.CommentRepository;
import np.something.repositories.PostRepository;
import np.something.util.validation.ValidatorUtil;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

@Service
public class PostService {
    private final PostRepository postRepository;
    private final CommentRepository commentRepository;
    private final ValidatorUtil validatorUtil;

    public PostService(PostRepository postRepository,
                       CommentRepository commentRepository, ValidatorUtil validatorUtil) {
        this.postRepository = postRepository;
        this.commentRepository = commentRepository;
        this.validatorUtil = validatorUtil;
    }

    @Transactional
    public Post findPost(Long id) {
        return postRepository.findById(id).orElseThrow(() -> new PostNotFoundException(id));
    }

    @Transactional
    public List<Post> findAllPosts() {
        return postRepository.findAll();
    }

    @Transactional
    public Post addPost(Customer customer, String title, String content) {
        Post post = new Post(customer, title, content);
        post.setCreateDate(LocalDateTime.now());
        validatorUtil.validate(post);
        customer.getPosts().add(post);
        return postRepository.save(post);
    }

    @Transactional
    public Post updatePost(Long id, String title, String content) {
        Post post = findPost(id);
        post.setTitle(title);
        post.setContent(content);
        validatorUtil.validate(post);
        return postRepository.save(post);
    }

    @Transactional
    public Post deletePost(Long id) {
        Post post = findPost(id);
        post.getCustomer().getPosts().remove(post);
        postRepository.delete(post);
        return post;
    }

    @Transactional
    public void deleteAllPosts() {
        postRepository.deleteAll();
    }

    @Transactional
    public List<PostDto> searchPosts(String search) {
        var posts = new ArrayList<>(postRepository.searchPosts(search));
        var comments = commentRepository.searchComments(search);
        for (var post: posts) {
            post.getComments().clear();
        }
        for (var comment: comments) {
            boolean found = false;
            for (var post: posts) {
                if (Objects.equals(comment.getPost().getId(), post.getId())) {
                    post.getComments().add(comment);
                    found = true;
                    break;
                }
            }
            if (!found) {
                var newPost = comment.getPost();
                newPost.getComments().clear();
                newPost.getComments().add(comment);
                posts.add(newPost);
            }
        }
        return posts.stream().map(PostDto::new).toList();
    }
}
